import axios, { AxiosRequestConfig, Method } from 'axios';

/* REQUEST_EXPORTS */

export const API = (method: Method, url: string, data: any = null) => {
  return axios.create({
    baseURL: process.env.REACT_APP_BASE_URL,
  }).request({ method, url, data } as AxiosRequestConfig);
};

export interface Request {
  status: 'idle' | 'loading' | 'success' | 'failed';
}

export interface RequestErrors {
  status: number;
  title: string;
  detail: string;
}

export interface FailedRequest {
  errors: RequestErrors;
}

export interface GetData<M> {
  type: string;
  id: number;
  attributes: M;
}

export interface GetAllRequest<M> {
  data: GetData<M>[];
}

export interface GetOneRequest<M> {
  data: GetData<M>;
}

export interface UpsertRelationshipsData<RL = null> {
  attributes?: RL | null;
}

export interface UpsertRelationshipsRequest<RL = null> {
  data?: UpsertRelationshipsData<RL> | null;
}

export interface UpsertData<RQ, RL = null> {
  attributes: RQ;
  relationships?: UpsertRelationshipsRequest<RL> | null;
}

export interface UpsertRequest<RQ, RL = null> {
  data: UpsertData<RQ, RL>;
}

export interface DeleteAttributes {
  success: boolean;
}

export interface DeleteData {
  attributes: DeleteAttributes;
}

export interface DeleteRequest {
  data: DeleteData;
}
